﻿<?php
/**
 * CosmicWord SEO Functionality
 *
 * @package CosmicWord
 * @subpackage SEO
 */

// Ensure this file is being included by CosmicWord
if (!defined('ABSPATH')) {
    exit;
}

class CosmicWord_SEO {
    private static $instance = null;

    private function __construct() {
        // Constructor
    }

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function init() {
        // Use CosmicWord's action system if available, otherwise these will need to be called manually
        $this->add_action('wp_head', array($this, 'output_seo_meta_tags'));
        $this->add_action('add_meta_boxes', array($this, 'add_seo_meta_box'));
        $this->add_action('save_post', array($this, 'save_seo_fields'));

        // New actions for managing columns and quick edit
        $this->add_action('manage_posts_columns', array($this, 'add_seo_columns'));
        $this->add_action('manage_pages_columns', array($this, 'add_seo_columns'));
        $this->add_action('manage_posts_custom_column', array($this, 'manage_seo_columns'), 10, 2);
        $this->add_action('manage_pages_custom_column', array($this, 'manage_seo_columns'), 10, 2);
        $this->add_action('quick_edit_custom_box', array($this, 'quick_edit_seo_fields'), 10, 2);
        $this->add_action('save_post', array($this, 'save_quick_edit_seo_fields'));
        $this->add_action('admin_footer', array($this, 'quick_edit_javascript'));
    }

    private function add_action($hook, $callback, $priority = 10, $accepted_args = 1) {
        // Check if CosmicWord has an add_action function
        if (function_exists('add_action')) {
            add_action($hook, $callback, $priority, $accepted_args);
        }
        // If not, you might need to implement your own action system or call these methods directly
    }

    public function output_seo_meta_tags() {
        global $post;
        if (is_singular() && $post) {
            $seo_title = $this->get_post_meta($post->ID, '_seo_title', true);
            $seo_description = $this->get_post_meta($post->ID, '_seo_description', true);
            $seo_keywords = $this->get_post_meta($post->ID, '_seo_keywords', true);
            $seo_canonical = $this->get_post_meta($post->ID, '_seo_canonical', true);

            if (!empty($seo_title)) {
                echo '<title>' . $this->esc_html($seo_title) . '</title>' . "\n";
                echo '<meta name="title" content="' . $this->esc_attr($seo_title) . '">' . "\n";
            }
            if (!empty($seo_description)) {
                echo '<meta name="description" content="' . $this->esc_attr($seo_description) . '">' . "\n";
            }
            if (!empty($seo_keywords)) {
                echo '<meta name="keywords" content="' . $this->esc_attr($seo_keywords) . '">' . "\n";
            }
            if (!empty($seo_canonical)) {
                echo '<link rel="canonical" href="' . $this->esc_url($seo_canonical) . '">' . "\n";
            }
        }
    }

    public function add_seo_meta_box() {
        add_meta_box(
            'cosmicword_seo',
            'SEO Settings',
            array($this, 'render_seo_meta_box'),
            array('post', 'page'),
            'normal',
            'high'
        );
    }

    public function render_seo_meta_box($post) {
        $seo_title = $this->get_post_meta($post->ID, '_seo_title', true);
        $seo_description = $this->get_post_meta($post->ID, '_seo_description', true);
        $seo_keywords = $this->get_post_meta($post->ID, '_seo_keywords', true);
        $seo_canonical = $this->get_post_meta($post->ID, '_seo_canonical', true);

        echo '<style>
            .cosmicword-seo-container {
                text-align: center;
                max-width: 600px;
                margin: 0 auto;
            }
            .cosmicword-seo-field {
                margin-bottom: 15px;
            }
            .cosmicword-seo-field label {
                display: block;
                margin-bottom: 5px;
                font-weight: bold;
            }
            .cosmicword-seo-field input[type="text"],
            .cosmicword-seo-field textarea {
                width: 100%;
                padding: 5px;
            }
        </style>';

        echo '<div class="cosmicword-seo-container">';
        
        echo '<div class="cosmicword-seo-field">';
        echo '<label for="seo_title">SEO Title:</label>';
        echo '<input type="text" id="seo_title" name="seo_title" value="' . $this->esc_attr($seo_title) . '">';
        echo '</div>';

        echo '<div class="cosmicword-seo-field">';
        echo '<label for="seo_description">SEO Description:</label>';
        echo '<textarea id="seo_description" name="seo_description" rows="4">' . $this->esc_textarea($seo_description) . '</textarea>';
        echo '</div>';

        echo '<div class="cosmicword-seo-field">';
        echo '<label for="seo_keywords">SEO Keywords:</label>';
        echo '<input type="text" id="seo_keywords" name="seo_keywords" value="' . $this->esc_attr($seo_keywords) . '">';
        echo '</div>';

        echo '<div class="cosmicword-seo-field">';
        echo '<label for="seo_canonical">Canonical URL:</label>';
        echo '<input type="text" id="seo_canonical" name="seo_canonical" value="' . $this->esc_url($seo_canonical) . '">';
        echo '</div>';

        echo '</div>'; // Close cosmicword-seo-container
    }

    public function save_seo_fields($post_id) {
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;

        $fields = array('seo_title', 'seo_description', 'seo_keywords', 'seo_canonical');
        foreach ($fields as $field) {
            if (isset($_POST[$field])) {
                $this->update_post_meta($post_id, '_' . $field, $this->sanitize_text_field($_POST[$field]));
            }
        }
    }

    public function add_seo_columns($columns) {
        $columns['seo_title'] = 'SEO Title';
        $columns['seo_description'] = 'SEO Description';
        $columns['seo_keywords'] = 'SEO Keywords';
        return $columns;
    }

    public function manage_seo_columns($column_name, $post_id) {
        switch ($column_name) {
            case 'seo_title':
                echo $this->get_post_meta($post_id, '_seo_title', true);
                break;
            case 'seo_description':
                echo $this->get_post_meta($post_id, '_seo_description', true);
                break;
            case 'seo_keywords':
                echo $this->get_post_meta($post_id, '_seo_keywords', true);
                break;
        }
    }

    public function quick_edit_seo_fields($column_name, $post_type) {
        if ($column_name != 'seo_title') return;
        ?>
        <fieldset class="inline-edit-col-right">
            <div class="inline-edit-col">
                <label>
                    <span class="title">SEO Title</span>
                    <span class="input-text-wrap"><input type="text" name="seo_title" class="ptitle" value=""></span>
                </label>
                <label>
                    <span class="title">SEO Description</span>
                    <span class="input-text-wrap"><textarea name="seo_description" rows="2"></textarea></span>
                </label>
                <label>
                    <span class="title">SEO Keywords</span>
                    <span class="input-text-wrap"><input type="text" name="seo_keywords" value=""></span>
                </label>
            </div>
        </fieldset>
        <?php
    }

    public function save_quick_edit_seo_fields($post_id) {
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;
        if (!current_user_can('edit_post', $post_id)) return;

        $fields = array('seo_title', 'seo_description', 'seo_keywords');
        foreach ($fields as $field) {
            if (isset($_POST[$field])) {
                $this->update_post_meta($post_id, '_' . $field, $this->sanitize_text_field($_POST[$field]));
            }
        }
    }

    public function quick_edit_javascript() {
        $screen = get_current_screen();
        if ('edit' !== $screen->base || !in_array($screen->post_type, array('post', 'page'))) return;
        ?>
        <script type="text/javascript">
        jQuery(function($) {
            var $wp_inline_edit = inlineEditPost.edit;
            inlineEditPost.edit = function(id) {
                $wp_inline_edit.apply(this, arguments);
                var $post_id = 0;
                if (typeof(id) == 'object') {
                    $post_id = parseInt(this.getId(id));
                }
                if ($post_id > 0) {
                    var $edit_row = $('#edit-' + $post_id);
                    var $post_row = $('#post-' + $post_id);
                    $edit_row.find('input[name="seo_title"]').val($post_row.find('.column-seo_title').text());
                    $edit_row.find('textarea[name="seo_description"]').val($post_row.find('.column-seo_description').text());
                    $edit_row.find('input[name="seo_keywords"]').val($post_row.find('.column-seo_keywords').text());
                }
            };
        });
        </script>
        <?php
    }

    private function get_post_meta($post_id, $key, $single = false) {
        // Implement based on how CosmicWord stores post meta
        // This is just a placeholder
        return get_post_meta($post_id, $key, $single);
    }

    private function update_post_meta($post_id, $key, $value) {
        // Implement based on how CosmicWord updates post meta
        // This is just a placeholder
        return update_post_meta($post_id, $key, $value);
    }

    private function esc_html($text) {
        return htmlspecialchars($text, ENT_QUOTES | ENT_HTML5, 'UTF-8');
    }

    private function esc_attr($text) {
        return htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
    }

    private function esc_textarea($text) {
        return htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
    }

    private function esc_url($url) {
        return filter_var($url, FILTER_SANITIZE_URL);
    }

    private function sanitize_text_field($text) {
        return trim(strip_tags($text));
    }
}

function cosmicword_seo_init() {
    CosmicWord_SEO::get_instance()->init();
}

// If CosmicWord has an 'init' action, use it; otherwise, you'll need to call this function manually
add_action('init', 'cosmicword_seo_init');